import functools

def trace_func(func):
    @functools.wraps(func)
    def wrapper_trace_func(*args, **kwargs):
        # Log function entry and arguments
        print(f"Entering {func.__name__} with args: {args}, kwargs: {kwargs}")
        result = func(*args, **kwargs)
        # Log function exit and result
        print(f"Exiting {func.__name__} with result: {result}")
        return result
    return wrapper_trace_func

if __name__ == '__main__':
    # below is the function to be traced, but it is not defined here, just a placeholder
    trace_func('dk')


# cpdec --> check package decorator

import subprocess
import functools
import os
import re

def check_package(package: str, venv_path: str):
    def decorator_check_package(func):
        @functools.wraps(func)
        def wrapper_check_package(*args, **kwargs):
            # Determine the correct activation script based on the OS
            if os.name == 'nt':  # Windows
                activate_script = f"{venv_path}\\Scripts\\activate.bat"
                command = f"cmd /c {activate_script} && pip list"
            else:  # Linux/Unix/Mac
                activate_script = f"{venv_path}/bin/activate"
                command = f"source {activate_script} && pip list"
            
            try:
                result = subprocess.run(command, shell=True, capture_output=True, text=True, executable='/bin/bash' if os.name != 'nt' else None)
                if result.returncode == 0:
                    installed_packages = result.stdout
                    package_pattern = re.compile(rf"{package.replace('==', '==|==').replace('<', '<|==').replace('>', '>|==')}\s.*", re.IGNORECASE)
                    if any(package_pattern.search(line) for line in installed_packages.split('\n')):
                        print(f"{package} is installed in the selected environment here ~ {venv_path}.")
                        return func(*args, **kwargs)
                    else:
                        print(f"{package} is NOT installed in the selected environment here ~ {venv_path}. Function '{func.__name__}' will not be executed.")
                        return None
                else:
                    print(f"Failed to list packages in the virtual environment: {result.stderr}")
                    return None
            except Exception as e:
                print(f"An error occurred: {str(e)}")
                return None

        return wrapper_check_package
    return decorator_check_package


if __name__ == '__main__':
    check_package()




import argparse
import sys

# import os

# # Hardcoded version information
VERSION = "0.0.2"

# # Hardcoded dependencies information
# DEPENDENCIES = {
#     "python": "^3.11",
#     "PyYAML": "^6.0.2",
#     "SQLAlchemy": "^2.0.36",
#     "psycopg2": "^2.9.10",
#     "pandas": "^2.2.3",
#     "pyodbc": "^5.2.0",
#     "pylint": "^3.3.1"
# }

# Hardcoded usage information
EXAMPLE_TRACE = """

sample usage:

from decoraid.trace import trace_func


# wrap the decorator around the function you want to trace
@trace_func
def test_pandas():
    print('test_trace')

if __name__ == "__main__":
    test_pandas()

"""

# Hardcoded usage information
EXAMPLE_CPDEC = """

sample usage:

from decoraid.cpdec import check_package
venv_path = "C:/projects/git/xxxx/.venv/" 

# wrap the decorator around the function you want to test, provide the name of the package and the path to the virtual environment
@check_package("pandas", venv_path)
def test_pandas():
    print('test_cpdec')

if __name__ == "__main__":
    test_pandas()

"""



def main():
    parser = argparse.ArgumentParser(description='Decoraid ~ collection of usefull decorator utilities')
    parser.add_argument('--version', action='version', version=f'ConnectionVault {VERSION}')
    parser.add_argument('--example', action='store_true', help='Show sample code syntax')
    parser.add_argument('-cpdec', type=str, help='Template example for cpdec usage')
    parser.add_argument('-trace', type=str, help='Template example for trace usage')

    
    args = parser.parse_args()

    # if args.example:
    #     print(EXAMPLE)

    if args.example:
        if not args.cpdec:
            print(" -cpdec is required for --example")
            sys.exit(1) 
        print(EXAMPLE_CPDEC) 
        sys.exit(0)

    if args.example:
        if not args.trace:
            print(" -trace is required for --example")
            sys.exit(1) 
        print(EXAMPLE_TRACE) 
        sys.exit(0)

    # if args.example:
    #     if not args.cpdec or not args.trace:
    #         print(" -cpdec or -trace is required for --example")
    #         sys.exit(1) 
    #     print(EXAMPLE_CPDEC) 
    #     sys.exit(0)



if __name__ == '__main__':
    main()